import pygame
from control import *
from data import filepath
import time

class Display:
  catalog = []
  
  def __init__(self,x,y,screen):
    self.controls = []
    self.x = x
    self.y = y
    self.screen = screen
    self.__class__.catalog.append(self)

class Monitor(Display):
  def __init__(self,x,y,screen,player):
    self.sizefont = 20
    self.font     = pygame.font.SysFont(None,self.sizefont)
    self.player   = player
    Display.__init__(self,x,y,screen)
  
  def write(self, text):
    y = self.y
    rects = []
    for t in text :
      s = self.font.render(t, True, (255,255,255))
      rects.append(self.screen.blit(s, (self.x,y)))
      y+=self.sizefont
    return rects

class WinLooseMonitor(Monitor):
  
  def __init__(self,x,y,screen,player) :
    self.time = (time.time() - player.time)/1000.
    Monitor.__init__(self,x,y,screen,player)
  
  def draw(self) :
    if self.player.loose :
      txt = ("You Loose !",)
    else :
      txt = ("Good Game !",)
    self.write(txt)


class BottomRightMonitor(Monitor):
  
  def __init__(self,x,y,screen,player):
    Monitor.__init__(self,x,y,screen,player)

  def draw(self):
    infect = "INFECTING: " + str(self.player.getInfecting())
    rects = self.write((infect,))
    if not self.controls :
      i=0
      for rect in rects :
        self.controls.append(BottomRightControl(rect,self.player,i))
        i+=1

class TopLeftMonitor(Monitor):
  def __init__(self,x,y,screen,player):
    Monitor.__init__(self,x,y,screen,player) 
  
  def draw(self):
    bp = str(int(self.player.availablebp)) + r" TBits/s"
    self.write((bp,))

class ListMonitor(Monitor):
  def __init__(self,x,y,screen,player,value):
    self.bottom = y
    self.status = False
    if value == "attacks" :
      self.get = player.getAttacks
    elif value == "evolutions" :
      self.get = player.getEvolutions
    self.rects = [pygame.Rect(0,0,0,0)]
    Monitor.__init__(self,x,y,screen,player)
      
  def draw(self):
    if self.status :
      self.y = self.bottom -len(self.get())*self.sizefont
      txt = [ a.name for a in self.get() ]
      self.rects = self.write(txt)
      i=0
      if not self.controls :
        for rect in self.rects :
          if self.get == self.player.getAttacks :
            self.controls.append(ListControl(rect,self.player,"attacks",i))
          elif self.get == self.player.getEvolutions :
            self.controls.append(ListControl(rect,self.player,"evolutions",i))
          i+=1
    else :
      for control in self.controls :
        Control.catalog.remove(control)
      self.controls = []
      self.rects = [pygame.Rect(0,0,0,0)]
  
class Menu(Display) :
  
  def __init__(self,x,y,screen,player):
    self.rects      = []
    self.attackicon = pygame.image.load(filepath("attack.png"))
    self.widthicon  = self.attackicon.get_width()
    self.height     = self.attackicon.get_height()
    self.monitors   = [ListMonitor(x,y,screen,player,"attacks")]
    self.icons      = [self.attackicon]
    Display.__init__(self,x,y,screen)
  
  def ismouseon(self,pos) :
    # For all the monitors
    for i in range(len(self.monitors)) :
      mon = self.monitors[i]
      # is it actif last round ?
      if mon.status == True :
        menuZone = pygame.Rect(mon.rects[0].left,mon.rects[0].top,self.height,self.y+self.x+(i+2)*self.widthicon)
        # is it still actif ?
        if menuZone.collidepoint(pos) :
          pass
        else :
          mon.status = False
      # are we in an other area ?
      elif self.rects[i].collidepoint(pos):
        mon.status = True

  def draw(self):
    x = self.x
    self.rects = []
    for icon in self.icons :
      self.rects.append(self.screen.blit(icon, (x,self.y)))
      x+=self.widthicon
