import pygame
from pygame.locals import *

import data

from map import Map
from structure import *
from player import Player
from display import *
from control import *
from random import random

class Game:
  def __init__(self):
    self.clock       = pygame.time.Clock()
    self.time_elapsed = self.clock.tick(40)
    self.size        = (640,480)
    self.screen      = pygame.display.set_mode(self.size)
    self.area        = self.screen.get_rect()
    # map
    self.map         = Map('map.svg')
    # structure
    self.home        = Home(574, 225, self.map)
    # structures
    for gov in self.map.govList :
      Government(gov[0], gov[1], self.map, False)
    for calc in self.map.calcList :
      CalcFarm(calc[0], calc[1], self.map, False)
    for router in self.map.routerList :
      Router(router[0], router[1], self.map, True)
    for wire in self.map.lineList :
      Wire(wire[0], wire[1], self.map)
    # player
    black            = 0,0,0
    flop             = 1
    bp               = 1 
    money            = 1
    spamming         = 0
    infecting        = 0
    evolving         = 0
    nextVersion      = 100000
    structures       = [self.home]
    self.player      = Player(flop, bp, money, spamming, infecting, evolving, structures, nextVersion)
    # monitors and controls
    TopLeftMonitor(10,10,self.screen,self.player)
    BottomRightMonitor(520,440,self.screen,self.player)
    self.menu = Menu(30,420,self.screen,self.player)
    self.menu.draw()
    
  def mainLoop(self):
    # time
    self.time_elapsed = self.clock.tick(40)
    # events
    for event in pygame.event.get() :
      if event.type == MOUSEBUTTONDOWN :
        pos = pygame.mouse.get_pos()
        self.attack(pos)
        for control in Control.catalog :
          control.click(pygame.mouse.get_pos(),event.button,10)
      if event.type == QUIT :
        return False
    # scroll
    if pygame.mouse.get_pos()[0] > (self.size[0]-40) and self.area.right < self.map.limit_x:
      self.area = self.area.move(10,0)
    if pygame.mouse.get_pos()[0] < 40 and self.area.left > 0:
      self.area = self.area.move(-10,0)
    # menu activation
    self.menu.ismouseon(pygame.mouse.get_pos())
    if self.player.loose or self.player.win :
      WinLooseMonitor(300,240,self.screen,self.player)
    else :
      self.update()
    # draw
    self.map.draw()
    for wire in Wire.catalog:
      wire.draw()
    for struct in Structure.catalog:
      struct.draw()
    self.screen.blit(self.map, (0,0), self.area)
    for display in Display.catalog :
      display.draw()
    # flip
    pygame.display.flip()
    return True
  
  def attack(self,pos):
    if self.player.selectedAttack :
      pos = pos[0] + self.area.left, pos[1] + self.area.top
      print pos
      for struct in Structure.catalog :
        if struct.collidepoint(pos) and type(struct) != Zombie and not struct.hidden and not struct.attack :
          if self.player.availablebp >= self.player.selectedAttack.bandwidth :
            struct.startAttack(self.player.selectedAttack)
      self.player.noAttack()
          

  def update(self):
    # Add zombie
    if random() < self.player.getProba()*(self.time_elapsed/1000.0) :
      Zombie(self.map,Structure.catalog)
    # Del zombie
    for struct in Structure.catalog:
      if struct.__class__ == Zombie :
        if random() < (1/50.)*(self.time_elapsed/1000.0) :
          if struct.structtarget :
            struct.structtarget.stopAttack()
          Structure.catalog.remove(struct)
    # collector
    coll = Collector()
    for struct in Structure.catalog:
      struct.update(coll)
    # apply collector changes (time factor)
    self.player.setMoney(coll.getMoney())
    self.player.setFlop(coll.getFlop())
    usedBp = 0
    for struct in Structure.catalog :
      if struct.attack :
        usedBp += struct.attack.bandwidth
    self.player.availablebp = coll.getBandwidth()*(100-self.player.getInfecting())/100 - usedBp
    for struct in coll.alerted :
      routers = [router for router in Structure.catalog if router.infected]
      lmin = 2000
      for router in routers :
        l = sqrt(pow((router.centerx-struct.centerx),2) + pow((router.centery-struct.centery),2))
        if l < lmin : 
          lmin=l
          routertouninefect = router
      routertouninefect.infected = False
      struct.alerted = 0
      coll.alerted.remove(struct)
      break
    # maj attack
    for struct in Structure.catalog :
      if struct.infected and (struct.__class__ == Router or struct.__class__ == Home)  :
        for wire in struct.wires :
          wire.hidden = False
      if struct.attacked :
        struct.attackInProgress(self.time_elapsed,self.player.availablebp)
    for wire in Wire.catalog :
      if wire.hidden == False :
        wire.routeur1.hidden = False;
        wire.routeur2.hidden = False;
    self.player.loose = coll.loose
    gov = [ gov for gov in Structure.catalog if type(gov) == Government ]
    win = True
    for g in gov :
      if not g.infected :
        win = False
    if win :
      self.player.win = True

class Collector:
  def __init__(self):
    self.loose = False
    self.alerted = []
    self.money = 0
    self.infected = 0
    self.bandwidth = 0
    self.flop = 0

  def display(self):
    print " loose : "+str(self.loose)+" bandwidth : "+str(self.bandwidth)+" alerted : "+str(self.alerted)+" money : "+str(self.money)+" infected : "+str(self.infected)+" flop : "+str(self.flop)

  def setLoose(self, loose):
    self.loose = loose
    return self.loose

  def getLoose(self):
    return self.loose

  def setAlerted(self, alerted):
    self.alerted = alerted
    return self.alerted

  def getAlerted(self):
    return self.alerted

  def addAlerted(self,alerted):
    self.alerted.append(alerted)

  def setMoney(self, money):
    self.money = money
    return self.money

  def getMoney(self):
    return self.money

  def addMoney(self,money):
    self.money += money

  def setInfected(self, infected):
    self.infected = infected
    return self.infected

  def addBandwidth(self,bandwidth):
    self.bandwidth += bandwidth

  def setBandwidth(self, bandwith):
    self.bandwith = bandwith
    return self.bandwidth

  def getBandwidth(self):
    return self.bandwidth

  def getInfected(self):
    return self.infected

  def addInfected(infected):
    self.infected += infected

  def setFlop(self, flop):
    self.flop = flop
    return self.flop

  def getFlop(self):
    return self.flop

  def addFlop(self,flop):
    self.flop += flop
