import pygame
from xml.dom import minidom
from xml import xpath
from data import load

class Map(pygame.Surface):

  def __init__(self, filename):
    self.limit_x = 1024
    self.limit_y = 480
    self.filename  = filename
    self.paths, self.accessibleZone, self.govList, self.calcList, self.routerList, self.lineList = self.generator(filename)
    self.accessibleZone.sort(lambda x, y: cmp(x[2], y[2]))
    pygame.Surface.__init__(self,(self.limit_x,self.limit_y))
    
  def draw(self):
    self.fill((0,0,0))
    for path in self.paths :
      pygame.draw.aalines(self, (30,144,255), False, path)
    #for circle in self.accessibleZone :
    #  pygame.draw.circle(self, (0,0,110), (circle[0],circle[1]), circle[2])

  def generator(self, filename):
    svgXML = minidom.parseString(load(filename).read())
    listPath = xpath.Evaluate("//svg/g/*",svgXML)
    paths = []
    accessibleZone = []
    govList = []
    calcList = []
    routerList = []
    lineList = []
    for path in listPath :
      index = 1
      # Structure
      if path.nodeName == "rect" :
        point = (int(float(path.getAttribute("x"))), int(float(path.getAttribute("y"))))
        width, height = (path.getAttribute("width"), path.getAttribute("height"))
        if width == "4" and height == "4" :
          routerList.append(point)
        if width == "2" and height == "2" :
          calcList.append(point)
        if width == "8" and height == "8" :
          govList.append(point)
      # Accessible Zone
      elif path.getAttribute("sodipodi:type") == "arc" :
        transform = path.getAttribute("transform")
        transform = transform[7:-1].split(",")
        cx, cy, rx = (path.getAttribute("sodipodi:cx"), path.getAttribute("sodipodi:cy"), path.getAttribute("sodipodi:rx"))
        cx = int(float(transform[4]) + float(transform[0]) * float(cx))
        cy = int(float(transform[5]) + float(transform[0]) * float(cy))
        rx = int(float(transform[0]) * float(rx))
        accessibleZone.append((cx, cy, rx))
      # Lines
      else :
        points = path.getAttribute("d").split(" ")
        # a path start
        border = []
        for point in points:
          if index == (index / 2) * 2 :
            x, y = point.split(",")
            point = (int(float(x)), int(float(y)))
            border.append(point)
          index = index + 1
        # a path end
        style = path.getAttribute("style").split(";")
        isLine = False
        # kind of line start
        for attribut in style :
          if attribut[0:len("stroke-width:2")] == "stroke-width:2" :
            isLine = True
        if isLine :
          # put the line in the correct order
          if border[0][0] > border[1][0] :
            lineList.append(((border[1][0],border[1][1]),(border[0][0],border[0][1])))
          else :
            lineList.append(border)
        else :
          paths.append(border)
        # kind of line end
    return paths, accessibleZone, govList, calcList, routerList, lineList
