# -*- coding : utf-8 -*-

import pygame
from pygame import Rect
from data import filepath
from random import randrange
from random import random
from math import sin, cos, acos, sqrt, pow

class Structure(Rect) :
  
  catalog = []
  
  def __init__(self, x, y, filename, map, hidden = True, infected = False) :
    self.attacked   = 0
    self.defense    = 100
    
    self.hidden       = hidden
    self.infected     = infected
    self.map          = map
    self.wires        = []
    self.attack       = False
    self.proba        = False
    self.circle       = False
    self.offender     = False
    self.structtarget = False
    self.surface           = pygame.image.load(filepath(filename+".png"))
    try :
      self.infectedsurface = pygame.image.load(filepath(filename+"i.png"))
    except :
      self.infectedsurface = pygame.image.load(filepath(filename+".png"))
    rect            = self.surface.get_rect()
    Rect.__init__(self, x, y, rect.width, rect.height)
    self.__class__.catalog.append(self)

  def draw(self):
    if not self.hidden:
      if self.circle :
        self.circle.draw()
      if self.infected :
        self.map.blit(self.infectedsurface, (self.x,self.y))
      else :
        self.map.blit(self.surface, (self.x,self.y))
    
  def update(self, modifications_collector):
    pass
  
  def addWire(self,wire):
    self.wires.append(wire)
  
  def startAttack(self,attack) :
    lmin=1000
    lmax=0
    structures = [struct for struct in Structure.catalog if (struct.infected and not struct.structtarget)]
    if attack.online :
      objets = [struct for struct in structures if type(struct) == Router or type(struct) == Home]
      structures = objets
    for struct in structures :
      l = sqrt(pow((struct.centerx-self.centerx),2) + pow((struct.centery-self.centery),2))
      if l>lmax :
        lmax = l
      if l<lmin :
        lmin = l
        offender = struct
    if attack.online :
      if lmin > attack.range and not [wire for wire in self.wires if wire.hidden == False]:
        self.stopAttack()
        return
      self.proba = attack.proba
      color = 0,255,0
    else :
      self.proba = (lmax-lmin)/lmax*attack.proba
      offender.structtarget = self
      color = 255,0,0
    self.offender = offender
    offender.circle = Circle(offender.map, offender.center,color)
    self.attacked = self.defense
    self.attack = attack
    self.circle = Circle(self.map, self.center,color)
    return self.center
  
  def stopAttack(self):
    if self.offender :
      self.offender.structtarget = False
      self.offender.circle = False
    self.offender = False
    self.attack = False
    self.attacked = 0
    self.circle = False
    
  def setInfected(self) :
    self.stopAttack()
    self.infected = True
  
  def attackInProgress(self,time,bp) :
    if bp < 0 :
      self.stopAttack()
      return
    mod = self.attack.power * time/1000.
    if self.attacked - mod <= 0 :
      if random() < self.proba :
        self.setInfected()
      else :
        gvt = [struct for struct in Structure.catalog if (type(struct)==Government and not struct.infected)]
        lmin = 2000
        for bat in gvt :
          l = sqrt(pow((bat.centerx-self.centerx),2) + pow((bat.centery-self.centery),2))
          if l < lmin : 
            lmin=l
            nextgvt = bat
        nextgvt.setAlerted(nextgvt.getAlerted()+1)
        self.stopAttack()
    else :
      self.attacked -= mod

class Router(Structure) :
  def __init__(self,x,y,map, hidden = True, bandwidth = 5) :
    self.filename = "rout"
    self.bandwidth = bandwidth
    Structure.__init__(self,x,y,self.filename,map, hidden)

  def setBandwidth(self, bandwidth) :
    self.bandwidth = bandwidth
    return bandwidth
    
  def getBandwidth(self) :
    return self.bandwidth

  def update(self, modifications_collector):
    if self.infected :
      modifications_collector.addBandwidth(self.bandwidth)

class Home(Router):
  def __init__(self,x,y,map) :
    Router.__init__(self, x, y, map)
    self.bandwidth = 5
    self.filename = "gvt"
    self.infected = True
    self.hidden = False

  def update(self, modifications_collector):
    modifications_collector.addBandwidth(self.bandwidth)
    if not self.infected :
      modifications_collector.setLoose(True)
    if self.infected :
      modifications_collector.setLoose(False)

class CalcFarm(Structure) :
  def __init__(self,x,y,map, hidden = True, flop = 1) :
    self.filename = "gvt"
    self.flop = flop
    Structure.__init__(self,x,y,self.filename,map, hidden)

  def setFlop(self, flop) :
    self.flop = flop
    return flop
    
  def getFlop(self) :
    return self.flop

  def update(self, modifications_collector):
    if self.infected :
      modifications_collector.addFlop(self.flop)

class Government(Structure) :
  def __init__(self,x,y,map, hidden = True, alerted = False) :
    self.filename = "gvt"
    self.alerted  = alerted
    Structure.__init__(self,x,y,self.filename,map, hidden)

  def setAlerted(self, alerted) :
    self.alerted = alerted
    return alerted
    
  def getAlerted(self) :
    return self.alerted

  def update(self, modifications_collector):
    if self.alerted >= 2 :
      modifications_collector.addAlerted(self)
  
class Zombie(Structure) :
  def __init__(self,map,catalog, money = 0.1, hidden = False) :
    width,height = 4, 4
    self.money = money
    self.filename = "zomb"
    
    pond = [ p[2]*p[2] for p in map.accessibleZone ]
    i,seuil,sum, tries = 0, 0 ,0 ,0
    for p in pond : sum += p
    proba = random()*sum
    while seuil < proba :
      seuil+=pond[i]
      i+=1
    zone = map.accessibleZone[i-1]
    teta = random()*acos(-1)*2
    while 1:
      tries+=1
      x = int(random()*zone[2]*cos(teta) + zone[0])
      y = int(random()*zone[2]*sin(teta) + zone[1])
      if pygame.Rect(x,y,width,height).collidelist(catalog) == -1 or tries > 100 :
        Structure.__init__(self,x,y,self.filename,map, hidden, True)
        break
    if tries > 100 :
      print "Too many zombies in the area"
  
  def setMoney(self, money) :
    self.money = money
    return money
    
  def getMoney(self) :
    return self.money

  def update(self, modifications_collector):
    if self.infected :
      modifications_collector.addMoney(self.money)

class Wire:
  catalog = []
  def __init__(self, point1, point2, map, hidden = True) :
    self.map     = map
    self.hidden  = hidden
    self.point1  = point1
    self.point2  = point2
    index1 = Rect(point1,(1,1)).collidelist(Structure.catalog)
    if index1 != -1 :
      structure1 = Structure.catalog[index1]
    if structure1 != -1 and (structure1.__class__ == Router or structure1.__class__ == Home) :
      self.routeur1 = structure1
      self.routeur1.addWire(self)
    else :
      print "Wire in the space !"
      self.routeur1 = point1
    index2 = Rect(point2,(1,1)).collidelist(Structure.catalog)
    if index2 != -1 :
      structure2 = Structure.catalog[index2]
    if structure2 != -1 and (structure2.__class__ == Router or structure2.__class__ == Home) :
      self.routeur2 = structure2
      self.routeur2.addWire(self)
    else :
      print "Wire in the space !"
      self.routeur2 = point2
    self.flashPoints = [FlashPoint(map, point1, point2, point1, 5), 
                        FlashPoint(map, point1, point2, point2, 5),
                        FlashPoint(map, point1, point2, point1, 4)]
    self.__class__.catalog.append(self)

  def draw(self):
    if not self.hidden :
      for points in self.flashPoints:
        points.draw()
      pygame.draw.aaline(self.map, (125, 204, 140), self.point1, self.point2)

class FlashPoint:
  def __init__(self, map, point1, point2, startpoint, speed = 4) :
    self.map      = map
    self.point1   = point1
    self.point2   = point2
    self.flash    = [startpoint[0], startpoint[1]]
    self.flashor  = False
    self.flashvi  = speed
    self.distance = self.distance(point1, point2)

  def draw(self):
    # move the flashy point
    if self.flashor :
      self.flash[0] = self.flash[0]+(self.point2[0]-self.point1[0])/self.distance*self.flashvi
      self.flash[1] = self.flash[1]+(self.point2[1]-self.point1[1])/self.distance*self.flashvi
    else :
      self.flash[0] = self.flash[0]+(self.point1[0]-self.point2[0])/self.distance*self.flashvi
      self.flash[1] = self.flash[1]+(self.point1[1]-self.point2[1])/self.distance*self.flashvi
    # change the sens of the move of the flashy point
    if self.flash[0] >= self.point2[0] or self.flash[0] >= self.point2[0] :
      self.flashor = False
    if self.flash[0] <= self.point1[0] or self.flash[0] <= self.point1[0] :
      self.flashor = True
    # display circle on the wire
    pygame.draw.circle(self.map, (125, 125, 125), (int(self.flash[0]),int(self.flash[1])), 1)

  def distance(self, point1, point2):
    X = self.point2[0]-self.point1[0]
    Y = self.point2[1]-self.point1[1]
    return sqrt( X*X + Y*Y)

class Circle:
  def __init__(self,map,pos,color=(255,0,0)):
    self.origradius = 20
    self.radius     = self.origradius
    self.pos = pos
    self.map = map
    self.color = color
  def draw(self):
    pygame.draw.circle(self.map, self.color , self.pos, self.radius, 1)
    if self.radius > 1 :
      self.radius-=1
    else :
      self.radius = self.origradius
    
